==> lam_6.5.6.orig.tar.gz: lam-6.5.6.orig/share/mpi/wpost.c <==
/*
 * Copyright 1998-2001, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
 *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
 * 
 * This file is part of the Notre Dame LAM implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted subject to the conditions specified in the
 * LICENSE file.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wpost.c,v 1.1 2001/02/03 20:51:49 arodrig6 Exp $
 *
 *	Function:	- post a window
 *	Accepts:	- group
 *			- assertion
 *			- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>
#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

/*@
MPI_Win_post - Starts an RMA exposure epoch for a local window

Input Parameters:
+ group -  group of origin processes (handle) 
. assertion -  program assertion (integer) 
- win -  window object (handle) 

Notes: 

Starts an RMA exposure epoch for the local window associated with
win. Only processes in group should access the window with RMA calls
on win during this epoch. Each process in group must issue a matching
call to MPI_WIN_START. MPI_WIN_POST does not block.

.N fortran
.N IMPI_MPI2

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_GROUP
.N MPI_ERR_EPOCH

.seealso MPI_Put(3), MPI_Get(3), MPI_Win_create(3), MPI_Win_start(3), MPI_Win_complete(3), MPI_Win_fence(3), MPI_Win_free(3), MPI_Win_get_group(3), MPI_Win_get_group(3), MPI_Win_wait(3)

.N WEB
  @*/

int MPI_Win_post(MPI_Group group, int assertion, MPI_Win win)
{
	int		rank;			/* rank in window */
	int		err;
	int		i;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINPOST);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	if (group == MPI_GROUP_NULL || group == MPI_GROUP_EMPTY) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_GROUP, 0)));

	}
/*
 * Check epoch.
 */
	if (win->w_flags & (LAM_WFEXPEPOCH | LAM_WFPOSTED)) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_EPOCH, 0)));
	}
/*
 * Send "i-post-u" messages.
 */
	win->w_flags |= LAM_WFPOSTED | LAM_WFNOOUT;
	win->w_ncomplete = group->g_nprocs;

	for (i = 0; i < group->g_nprocs; i++) {
		rank = lam_rank_in_group(win->w_comm->c_group,
						group->g_procs[i]);

		if (rank == MPI_UNDEFINED) {
			return(lam_errfunc(win->w_comm,
				BLKMPIWINPOST, lam_mkerr(MPI_ERR_GROUP, 0)));
		}

		win->w_pstate[rank] |= LAM_WFPOSTED;

		err = lam_osd_send(0, 0, MPI_BYTE, rank,
				LAM_OSD_POST, win->w_comm, TRUE, FALSE);

		if (err != MPI_SUCCESS) {
			return(lam_errfunc(win->w_comm, BLKMPIWINPOST, err));
		}
	}

        lam_resetfunc_m(BLKMPIWINPOST);
	return(MPI_SUCCESS);
}

==> lam_7.0.6.orig.tar.gz: lam-7.0.6/share/mpi/wpost.c <==
/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: wpost.c,v 1.3.2.1 2003/08/21 06:32:00 brbarret Exp $
 *
 *	Function:	- post a window
 *	Accepts:	- group
 *			- assertion
 *			- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>
#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

/*@
MPI_Win_post - Starts an RMA exposure epoch for a local window

Input Parameters:
+ group -  group of origin processes (handle) 
. assertion -  program assertion (integer) 
- win -  window object (handle) 

Notes: 

Starts an RMA exposure epoch for the local window associated with
win. Only processes in group should access the window with RMA calls
on win during this epoch. Each process in group must issue a matching
call to MPI_WIN_START. MPI_WIN_POST does not block.

.N fortran
.N IMPI_MPI2

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_GROUP
.N MPI_ERR_EPOCH

.seealso MPI_Put(3), MPI_Get(3), MPI_Win_create(3), MPI_Win_start(3), MPI_Win_complete(3), MPI_Win_fence(3), MPI_Win_free(3), MPI_Win_get_group(3), MPI_Win_get_group(3), MPI_Win_wait(3)

.N WEB
  @*/

int MPI_Win_post(MPI_Group group, int assertion, MPI_Win win)
{
	int		rank;			/* rank in window */
	int		err;
	int		i;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINPOST);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_WIN, EINVAL)));
	}

	if (group == MPI_GROUP_NULL || group == MPI_GROUP_EMPTY) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_GROUP, EINVAL)));

	}
/*
 * Check epoch.
 */
	if (win->w_flags & (LAM_WFEXPEPOCH | LAM_WFPOSTED)) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_EPOCH, EINVAL)));
	}
/*
 * Send "i-post-u" messages.
 */
	win->w_flags |= LAM_WFPOSTED | LAM_WFNOOUT;
	win->w_ncomplete = group->g_nprocs;

	for (i = 0; i < group->g_nprocs; i++) {
		rank = lam_mpi_rank_in_group(win->w_comm->c_group,
						group->g_procs[i]);

		if (rank == MPI_UNDEFINED) {
			return(lam_errfunc(win->w_comm,
					   BLKMPIWINPOST, 
					   lam_mkerr(MPI_ERR_GROUP, EINVAL)));
		}

		win->w_pstate[rank] |= LAM_WFPOSTED;

		err = lam_osd_send(0, 0, MPI_BYTE, rank,
				LAM_OSD_POST, win->w_comm, TRUE, FALSE);

		if (err != MPI_SUCCESS) {
			return(lam_errfunc(win->w_comm, BLKMPIWINPOST, err));
		}
	}

        lam_resetfunc_m(BLKMPIWINPOST);
	return(MPI_SUCCESS);
}
